@@ -25,6 +25,8 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+import gi
+gi.require_version('Tracker', '1.0')
 from gi.repository import Tracker
 from itertools import chain
 import logging
@@ -41,6 +43,9 @@ def log(fn):
         global tabbing
         name = fn.__name__
         module = fn.__module__
+        filename = fn.__code__.co_filename.split('/')[-1]
+        lineno = fn.__code__.co_firstlineno
+
         params = ", ".join(map(repr, chain(v, k.values())))
 
         tabbing += 1
@@ -51,32 +56,13 @@ def log(fn):
         elapsed_time = ''
         if elapsed > 0.5:
             elapsed_time = ', took %02f' % elapsed
-        logger.debug("%s%s.%s(%s), returned %s%s", '|' * tabbing, module, name, params, retval, elapsed_time)
+        logger.debug("%s:%s\t%s%s.%s(%s), returned %s%s",
+                     filename, lineno, '|' * tabbing, module, name, params, retval, elapsed_time)
 
         return retval
     return wrapped
 
 
-def RateLimited(maxPerSecond):
-    minInterval = 1.0 / float(maxPerSecond)
-
-    def decorate(func):
-        lastTimeCalled = [0.0]
-
-        def rateLimitedFunction(*args, **kargs):
-            elapsed = time.clock() - lastTimeCalled[0]
-            leftToWait = minInterval - elapsed
-            if leftToWait > 0:
-                time.sleep(leftToWait)
-            ret = func(*args, **kargs)
-            lastTimeCalled[0] = time.clock()
-            return ret
-
-        return rateLimitedFunction
-
-    return decorate
-
-
 class TrackerWrapper:
     class __TrackerWrapper:
         def __init__(self):
