@@ -27,8 +27,9 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-
-from gi.repository import Gtk, GdkPixbuf, Gio, GLib, Gdk, MediaArt
+import gi
+gi.require_version('MediaArt', '2.0')
+from gi.repository import Gtk, GdkPixbuf, Gio, GLib, Gdk, MediaArt, GObject
 from gettext import gettext as _
 import cairo
 from math import pi
@@ -38,6 +39,7 @@ from gnomemusic import log
 from gnomemusic.grilo import grilo
 import logging
 from queue import Queue
+import urllib.request
 logger = logging.getLogger(__name__)
 
 WORKER_THREADS = 2
@@ -78,11 +80,15 @@ def _make_icon_frame(pixbuf, path=None):
     return border_pixbuf
 
 
-class AlbumArtCache:
+class AlbumArtCache(GObject.GObject):
     instance = None
     blacklist = {}
     itr_queue = []
     threading_lock = Lock()
+    default_icons_cache = {}
+
+    default_icon_width = 256
+    default_icon_height = 256
 
     @classmethod
     def get_default(self):
@@ -128,6 +134,7 @@ class AlbumArtCache:
 
     @log
     def __init__(self):
+        GObject.GObject.__init__(self)
         try:
             self.cacheDir = os.path.join(GLib.get_user_cache_dir(), 'media-art')
             if not os.path.exists(self.cacheDir):
@@ -135,6 +142,10 @@ class AlbumArtCache:
         except Exception as e:
             logger.warn("Error: %s", e)
 
+        # Prepare default icons
+        self.make_default_icon(is_loading=False)
+        self.make_default_icon(is_loading=True)
+
         try:
             self.thread_queue = Queue()
             for i in range(WORKER_THREADS):
@@ -144,8 +155,9 @@ class AlbumArtCache:
         except Exception as e:
             logger.warn("Error: %s", e)
 
-    @log
-    def get_default_icon(self, width, height, is_loading=False):
+    def make_default_icon(self, is_loading=False):
+        width = self.default_icon_width
+        height = self.default_icon_height
         # get a small pixbuf with the given path
         icon_name = 'folder-music-symbolic'
         if is_loading:
@@ -168,7 +180,32 @@ class AlbumArtCache:
                        icon.get_height() * 3 / 2,
                        1, 1,
                        GdkPixbuf.InterpType.NEAREST, 0x33)
-        return _make_icon_frame(result)
+        final_icon = _make_icon_frame(result)
+        if width not in self.default_icons_cache:
+            self.default_icons_cache[width] = {}
+        if height not in self.default_icons_cache[width]:
+            self.default_icons_cache[width][height] = {}
+        self.default_icons_cache[width][height][is_loading] = final_icon
+
+    @log
+    def get_default_icon(self, width, height, is_loading=False):
+        # Try to fetch the icon from cache
+        try:
+            return self.default_icons_cache[width][height][is_loading]
+        except:
+            pass
+
+        # Scale the image down
+        orig_icon = self.default_icons_cache[self.default_icon_width][self.default_icon_height][is_loading].copy()
+        final_icon = orig_icon.scale_simple(width, height, GdkPixbuf.InterpType.BILINEAR)
+
+        # Create a cache reference
+        if width not in self.default_icons_cache:
+            self.default_icons_cache[width] = {}
+        if height not in self.default_icons_cache[width]:
+            self.default_icons_cache[width][height] = {}
+        self.default_icons_cache[width][height][is_loading] = final_icon
+        return final_icon
 
     @log
     def lookup(self, item, width, height, callback, itr, artist, album):
@@ -271,7 +308,12 @@ class AlbumArtCache:
         try:
             src = Gio.File.new_for_uri(uri)
             dest = Gio.File.new_for_path(path)
-            src.copy(dest, Gio.FileCopyFlags.OVERWRITE)
+            try:
+                # First lets use GLib
+                src.copy(dest, Gio.FileCopyFlags.OVERWRITE)
+            except Exception as e:
+                # Try the native python way
+                urllib.request.urlretrieve(uri, path)
             self.lookup_worker(item, width, height, callback, itr, artist, album)
         except Exception as e:
             logger.warn("Error: %s", e)
