@@ -24,9 +24,12 @@
 # modify this code, you may extend this exception to your version of the
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
+
+import gi
+gi.require_version('Grl', '0.2')
 from gi.repository import GLib, GObject
 from gnomemusic.query import Query
-from gnomemusic import log, RateLimited, TrackerWrapper
+from gnomemusic import log, TrackerWrapper
 import logging
 import os
 os.environ['GRL_PLUGIN_RANKS'] = 'local-metadata:3,filesystem:2,tracker:1,lastfm-albumart:0'
@@ -37,9 +40,9 @@ logger = logging.getLogger(__name__)
 class Grilo(GObject.GObject):
 
     __gsignals__ = {
-        'ready': (GObject.SIGNAL_RUN_FIRST, None, ()),
-        'changes-pending': (GObject.SIGNAL_RUN_FIRST, None, ()),
-        'new-source-added': (GObject.SIGNAL_RUN_FIRST, None, (Grl.Source, ))
+        'ready': (GObject.SignalFlags.RUN_FIRST, None, ()),
+        'changes-pending': (GObject.SignalFlags.RUN_FIRST, None, ()),
+        'new-source-added': (GObject.SignalFlags.RUN_FIRST, None, (Grl.Source, ))
     }
 
     METADATA_KEYS = [
@@ -80,6 +83,7 @@ class Grilo(GObject.GObject):
         self.changed_media_ids = []
         self.pending_event_id = 0
         self.changes_pending = {'Albums': False, 'Artists': False, 'Songs': False}
+        self.pending_changed_medias = []
 
         self.registry = Grl.Registry.get_default()
 
@@ -97,8 +101,13 @@ class Grilo(GObject.GObject):
         if self.tracker is not None:
             logger.debug("tracker found")
 
+    def _rate_limited_content_changed(self, mediaSource, changedMedias, changeType, locationUnknown):
+        [self.pending_changed_medias.append(media) for media in changedMedias]
+        if self.content_changed_timeout is None:
+            self.content_changed_timeout = GLib.timeout_add(
+                500, self._on_content_changed, mediaSource, self.pending_changed_medias, changeType, locationUnknown)
+
     @log
-    @RateLimited(2)
     def _on_content_changed(self, mediaSource, changedMedias, changeType, locationUnknown):
         try:
             with self.tracker.handler_block(self.notification_handler):
@@ -121,7 +130,12 @@ class Grilo(GObject.GObject):
                             logger.warn("Skipping %s", media)
 
                 if self.changed_media_ids == []:
-                    return
+                    self.pending_changed_medias = []
+                    if self.content_changed_timeout is not None:
+                        GLib.source_remove(self.content_changed_timeout)
+                        self.content_changed_timeout = None
+                    return False
+
                 self.changed_media_ids = list(set(self.changed_media_ids))
                 logger.debug("Changed medias: %s", self.changed_media_ids)
 
@@ -134,6 +148,12 @@ class Grilo(GObject.GObject):
                     self.pending_event_id = GLib.timeout_add(self.CHANGED_MEDIA_SIGNAL_TIMEOUT, self.emit_change_signal)
         except Exception as e:
             logger.warn("Exception in _on_content_changed: %s", e)
+        finally:
+            self.pending_changed_medias = []
+            if self.content_changed_timeout is not None:
+                GLib.source_remove(self.content_changed_timeout)
+                self.content_changed_timeout = None
+            return False
 
     @log
     def emit_change_signal(self):
@@ -162,8 +182,9 @@ class Grilo(GObject.GObject):
                     if self.tracker is not None:
                         self.emit('ready')
                         self.tracker.notify_change_start()
+                        self.content_changed_timeout = None
                         self.notification_handler = self.tracker.connect(
-                            'content-changed', self._on_content_changed)
+                            'content-changed', self._rate_limited_content_changed)
 
             elif (id.startswith('grl-upnp')):
                 logger.debug("found upnp source %s", id)
@@ -243,10 +264,16 @@ class Grilo(GObject.GObject):
     @log
     def search(self, q, callback, data=None):
         options = self.options.copy()
+        self._search_callback_counter = 0
 
         @log
         def _search_callback(source, param, item, remaining, data, error):
             callback(source, param, item, remaining, data)
+            self._search_callback_counter += 1
+
+        @log
+        def _multiple_search_callback(source, param, item, remaining, data, error):
+            callback(source, param, item, remaining, data)
 
         if self.search_source:
             if self.search_source.get_id().startswith('grl-upnp'):
@@ -257,7 +284,7 @@ class Grilo(GObject.GObject):
             Grl.multiple_search([self.sources[key] for key in self.sources
                                  if key != 'grl-tracker-source'],
                                 q, self.METADATA_KEYS, options,
-                                _search_callback, data)
+                                _multiple_search_callback, data)
 
     @log
     def get_album_art_for_item(self, item, callback, data=None):
