@@ -28,8 +28,7 @@
 # delete this exception statement from your version.
 
 
-from gi.repository import Gtk, GObject, GLib
-from gettext import gettext as _
+from gi.repository import Gtk, GObject
 
 from gnomemusic.searchbar import Searchbar, DropDown
 
@@ -47,8 +46,7 @@ class ToolbarState:
 class Toolbar(GObject.GObject):
 
     __gsignals__ = {
-        'state-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
-        'selection-mode-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
+        'selection-mode-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
     _selectionMode = False
 
@@ -128,7 +126,6 @@ class Toolbar(GObject.GObject):
     def set_state(self, state, btn=None):
         self._state = state
         self._update()
-        self.emit('state-changed')
 
     @log
     def _update(self):
