@@ -36,7 +36,6 @@ from gi.repository import GObject
 from gi.repository import Gd
 from gi.repository import Gio
 from gi.repository import Grl
-from gi.repository import Gio
 from gi.repository import Pango
 from gi.repository import GLib
 from gi.repository import GdkPixbuf
@@ -133,8 +132,6 @@ class ViewContainer(Gtk.Stack):
 
         self._init = False
         grilo.connect('ready', self._on_grilo_ready)
-        self.header_bar.header_bar.connect('state-changed',
-                                           self._on_state_changed)
         self.header_bar.connect('selection-mode-changed',
                                 self._on_selection_mode_changed)
         self.view.connect('view-selection-changed',
@@ -197,10 +194,6 @@ class ViewContainer(Gtk.Stack):
         self._init = True
         self.populate()
 
-    @log
-    def _on_state_changed(self, widget, data=None):
-        pass
-
     @log
     def _on_selection_mode_changed(self, widget, data=None):
         pass
@@ -317,7 +310,6 @@ class Albums(ViewContainer):
         if (self._init and self.header_bar._selectionMode is False):
             self._offset = 0
             self._init = True
-            self.model.clear()
             self.populate()
             grilo.changes_pending['Albums'] = False
 
@@ -638,20 +630,9 @@ class Artists (ViewContainer):
     @log
     def _populate(self, data=None):
         selection = self.view.get_generic_view().get_selection()
-        if not selection.get_selected()[1]:
-            self._allIter = self.model.insert_with_valuesv(-1, [2], [_("All Artists")])
-            self._last_selection = self._allIter
-            self._artists[_("All Artists").casefold()] =\
-                {'iter': self._allIter, 'albums': [], 'widget': None}
-            #selection.select_path(self.model.get_path(self._allIter))
         self._init = True
         self.populate()
 
-    @log
-    def add_all_artists_entry(self):
-        self.view.emit('item-activated', '0',
-                       self.model.get_path(self._allIter))
-
     @log
     def _add_list_renderers(self):
         list_widget = self.view.get_generic_view()
@@ -705,16 +686,11 @@ class Artists (ViewContainer):
         self.artistAlbumsStack.add(new_artistAlbumsWidget)
 
         artistAlbums = None
-        if (self.model.get_string_from_iter(_iter) ==
-                self.model.get_string_from_iter(self._allIter)):
-            artistAlbums = Widgets.AllArtistsAlbums(
-                self.player, self.header_bar, self.selection_toolbar, self.window
-            )
-        else:
-            artistAlbums = Widgets.ArtistAlbums(
-                artist, albums, self.player,
-                self.header_bar, self.selection_toolbar, self.window
-            )
+
+        artistAlbums = Widgets.ArtistAlbums(
+		    artist, albums, self.player,
+		    self.header_bar, self.selection_toolbar, self.window
+        )
         self._artists[artist.casefold()]['widget'] = artistAlbums
         new_artistAlbumsWidget.add(artistAlbums)
         new_artistAlbumsWidget.show()
@@ -782,9 +758,7 @@ class Artists (ViewContainer):
             _iter = self.model.get_iter(path)
             artist = self.model.get_value(_iter, 2)
             albums = self._artists[artist.casefold()]['albums']
-            if (self.model.get_string_from_iter(_iter) !=
-                    self.model.get_string_from_iter(self._allIter)):
-                self.albums_selected.extend(albums)
+            self.albums_selected.extend(albums)
 
         if len(self.albums_selected):
             self._get_selected_album_songs()
@@ -809,8 +783,8 @@ class Artists (ViewContainer):
 
 class Playlist(ViewContainer):
     __gsignals__ = {
-        'playlists-loaded': (GObject.SIGNAL_RUN_FIRST, None, ()),
-        'playlist-songs-loaded': (GObject.SIGNAL_RUN_FIRST, None, ()),
+        'playlists-loaded': (GObject.SignalFlags.RUN_FIRST, None, ()),
+        'playlist-songs-loaded': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     @log
@@ -1322,7 +1296,7 @@ class EmptySearch(ViewContainer):
         widget = builder.get_object('container')
         widget.set_vexpand(True)
         widget.set_hexpand(True)
-        widget.get_children()[1].get_children()[1].set_text("Try a different search")
+        widget.get_children()[1].get_children()[1].set_text(_("Try a different search"))
         widget.show_all()
         self._box.add(widget)
 
@@ -1337,7 +1311,7 @@ class EmptySearch(ViewContainer):
 
 class Search(ViewContainer):
     __gsignals__ = {
-        'no-music-found': (GObject.SIGNAL_RUN_FIRST, None, ())
+        'no-music-found': (GObject.SignalFlags.RUN_FIRST, None, ())
     }
 
     @log
@@ -1372,11 +1346,13 @@ class Search(ViewContainer):
         self.items_selected = []
         self.items_selected_callback = None
 
+        self.found_items_number = None
+
     @log
     def _no_music_found_callback(self, view):
         self.window._stack.set_visible_child_name('emptysearch')
-        self.window._stack.get_child_by_name('emptysearch')._artistAlbumsWidget =\
-	        self._artistAlbumsWidget
+        emptysearch = self.window._stack.get_child_by_name('emptysearch')
+        emptysearch._artistAlbumsWidget = self._artistAlbumsWidget
 
     @log
     def _back_button_clicked(self, widget, data=None):
@@ -1441,7 +1417,12 @@ class Search(ViewContainer):
 
     @log
     def _add_search_item(self, source, param, item, remaining=0, data=None):
-        if not item or data != self.model:
+        if not item:
+            if grilo._search_callback_counter == 0 and grilo.search_source:
+                self.emit('no-music-found')
+            return
+
+        if data != self.model:
             return
 
         artist = item.get_string(Grl.METADATA_KEY_ARTIST) \
@@ -1469,6 +1450,17 @@ class Search(ViewContainer):
         if data is None:
             return
 
+        model, category = data
+
+        self.found_items_number = (self.model.iter_n_children(self.head_iters[0])+
+            self.model.iter_n_children(self.head_iters[1])+
+            self.model.iter_n_children(self.head_iters[2])+
+            self.model.iter_n_children(self.head_iters[3]))
+
+        if category == 'song' and self.found_items_number == 0 and remaining == 0:
+            if grilo.search_source:
+                self.emit('no-music-found')
+
         # We need to remember the view before the search view
         if self.window.curr_view != self.window.views[5] and \
            self.window.prev_view != self.window.views[5]:
@@ -1478,13 +1470,6 @@ class Search(ViewContainer):
             self.window.notification.dismiss()
             self.view.show()
 
-        if (self.model.iter_n_children(self.head_iters[0])+
-                self.model.iter_n_children(self.head_iters[1])+
-                self.model.iter_n_children(self.head_iters[2])+
-                self.model.iter_n_children(self.head_iters[3]) == 0) and remaining == 0:
-            self.emit('no-music-found')
-
-        model, category = data
         if not item or model != self.model:
             return
 
