@@ -29,9 +29,9 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-
-from gi.repository import Gtk, Gdk, Gio, GLib
-from gi.repository import Gd
+import gi
+gi.require_version('Gd', '1.0')
+from gi.repository import Gtk, Gdk, Gio, GLib, Gd
 from gettext import gettext as _, ngettext
 
 from gnomemusic import TrackerWrapper
@@ -42,7 +42,7 @@ import gnomemusic.view as Views
 import gnomemusic.widgets as Widgets
 from gnomemusic.playlists import Playlists
 from gnomemusic.grilo import grilo
-from gnomemusic import log, RateLimited
+from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
 
@@ -88,8 +88,9 @@ class Window(Gtk.ApplicationWindow):
 
         self._setup_view()
 
+        self.window_size_update_timeout = None
         self.connect("window-state-event", self._on_window_state_event)
-        self.configure_event_handler = self.connect("configure-event", self._on_configure_event)
+        self.connect("configure-event", self._on_configure_event)
 
         self.proxy = Gio.DBusProxy.new_sync(Gio.bus_get_sync(Gio.BusType.SESSION, None),
                                             Gio.DBusProxyFlags.NONE,
@@ -132,17 +133,19 @@ class Window(Gtk.ApplicationWindow):
                 self.toolbar.show_stack()
 
     def _on_configure_event(self, widget, event):
-        with self.handler_block(self.configure_event_handler):
-            GLib.idle_add(self.store_window_size_and_position, widget, priority=GLib.PRIORITY_LOW)
+        if self.window_size_update_timeout is None:
+            self.window_size_update_timeout = GLib.timeout_add(500, self.store_window_size_and_position, widget)
 
     @log
-    @RateLimited(2)
     def store_window_size_and_position(self, widget):
         size = widget.get_size()
         self.settings.set_value('window-size', GLib.Variant('ai', [size[0], size[1]]))
 
         position = widget.get_position()
         self.settings.set_value('window-position', GLib.Variant('ai', [position[0], position[1]]))
+        GLib.source_remove(self.window_size_update_timeout)
+        self.window_size_update_timeout = None
+        return False
 
     @log
     def _on_window_state_event(self, widget, event):
@@ -322,7 +325,7 @@ class Window(Gtk.ApplicationWindow):
     def _init_loading_notification(self):
         self.notification = Gd.Notification()
         self.notification.set_timeout(5)
-        grid = Gtk.Grid(valign=Gtk.Align.CENTER, margin_right=8)
+        grid = Gtk.Grid(valign=Gtk.Align.CENTER, margin_end=8)
         grid.set_column_spacing(8)
         spinner = Gtk.Spinner()
         spinner.start()
@@ -445,9 +448,9 @@ class Window(Gtk.ApplicationWindow):
     def _on_search_toggled(self, button, data=None):
         self.toolbar.searchbar.show_bar(button.get_active(),
                                         self.curr_view != self.views[4])
-        if not button.get_active() and \
-           	(self.curr_view == self.views[4] or self.curr_view == self.views[5])and \
-           self.toolbar._state == ToolbarState.MAIN:
+        if (not button.get_active() and
+                (self.curr_view == self.views[4] or self.curr_view == self.views[5]) and
+                self.toolbar._state == ToolbarState.MAIN):
             # We should get back to the view before the search
             self._stack.set_visible_child(self.views[4].previous_view)
             if self.toolbar._selectionMode:
